# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Flask-Resources is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Version information for Flask-Resources.

This file is imported by ``flask_resources.__init__``,
and parsed by ``setup.py``.
"""

__version__ = "0.5.0"
